#include "stdafx.h"
#include "resource.h"
#include "ConvertingDialog.hpp"
#include "Resource.hpp"
#include "Application.hpp"
#include "Utility.hpp"
#include "O2JamMusic.hpp"
#include "BmsConverter.hpp"
#include "ImageConverter.hpp"
#include "MusicPackage.hpp"
#include "SoundRanking.hpp"
#include "VorbisSoundConverter.hpp"
#include "WaveSoundConverter.hpp"
#include "AlphaDecimal.hpp"

namespace Lmt {
namespace O2Bms {
	ConvertingDialog::ConvertingDialog() : ModalDialog( IDD_CONVERTING ) {
		m_outputDir = L"";
		m_format = SoundFormat::Wave;
		m_quality = MiddleWaveQuality;
		//08/4/20 1.05----->
		m_diffType = O2JamDifficulty;
		//<-----------------
		//2007/5/20 1.02----->
		m_maxLevel = 1;
		m_maxLevelO2 = 1;
		//<-------------------
		m_judgeRank = NormalJudgement;
		m_outputEasy = false;
		m_outputNormal = false;
		m_outputHard = false;
		m_limitedSoundCount = false;
		//2007/2/24 1.01----->
		m_nothingLongNote = false;
		//<-------------------
	}

	//2007/5/23 1.03----->
//	void ConvertingDialog::AddTarget( const wstring &target ) {
	void ConvertingDialog::AddTarget( const ConvertTargetFile &target ) {
	//<-------------------
		m_targets.push_back( target );
		m_results.push_back( ConvertResult( ConvertResult::Canceled ) );
	}

	void ConvertingDialog::SetOutputDir( const wstring &outputDir ) {
		m_outputDir = outputDir;
	}

	void ConvertingDialog::SetSoundFormat( SoundFormat format ) {
		m_format = format;
	}

	void ConvertingDialog::SetWaveQualitiy( WaveQualities quality ) {
		m_quality = quality;
	}

	//08/4/20 1.05----->
	void ConvertingDialog::SetDifficultyType( DifficultyTypes diffType ) {
		m_diffType = diffType;
	}
	//<-----------------

	void ConvertingDialog::SetMaxLevel( int maxLevel ) {
		m_maxLevel = maxLevel;
	}

	//2007/5/20 1.02----->
	void ConvertingDialog::SetO2MaxLevel( int maxLevel ) {
		m_maxLevelO2 = maxLevel;
	}
	//<-------------------

	void ConvertingDialog::SetJudgeRank( JudgeRanks judgeRank ) {
		m_judgeRank = judgeRank;
	}

	void ConvertingDialog::SetOutputEasy( bool output ) {
		m_outputEasy = output;
	}

	void ConvertingDialog::SetOutputNormal( bool output ) {
		m_outputNormal = output;
	}

	void ConvertingDialog::SetOutputHard( bool output ) {
		m_outputHard = output;
	}

	void ConvertingDialog::SetLimitedSoundCount( bool limitedSoundCount ) {
		m_limitedSoundCount = limitedSoundCount;
	}

	//2007/2/24 1.01----->
	void ConvertingDialog::SetNothingLongNote( bool nothingLongNote ) {
		m_nothingLongNote = nothingLongNote;
	}
	//<-------------------

	//2007/5/24 1.03----->
	void ConvertingDialog::SetDirExistsAbort( bool dirExistsAbort ) {
		m_dirExistsAbort = dirExistsAbort;
	}
	//<-------------------

	int ConvertingDialog::GetTargetCount() const {
		return static_cast<int>( m_targets.size() );
	}

	//2007/5/23 1.03----->
//	wstring ConvertingDialog::GetTarget( int index ) const {
	ConvertTargetFile ConvertingDialog::GetTarget( int index ) const {
	//<-------------------
		return m_targets[index];
	}

	ConvertResult ConvertingDialog::GetConvertResult( int index ) const {
		return m_results[index];
	}

	bool ConvertingDialog::DialogProc( UINT msg, WPARAM wp, LPARAM lp ) {
		switch (msg) {
			case StartConvertSign:
				this->OnStartConvert();
				return true;
			case EndDialogSign:
				if (m_thread != NULL) {
					::WaitForSingleObject( m_thread, INFINITE );
					::CloseHandle( m_thread );
				}
				::EndDialog( m_hwnd, IDOK );
				return true;
			case UpdateProgressSign:
				//2007/5/23 1.03----->
//				::SetWindowText( this->GetItem( IDC_PROG_STATIC_INFO ), m_targets[wp].c_str() );
				::SetWindowText( this->GetItem( IDC_PROG_STATIC_INFO ), m_targets[wp].GetPath().c_str() );
				//<-------------------
				::SendMessage( this->GetItem( IDC_PROG_NOW ), PBM_SETPOS, wp, 0 );
				return true;
			case ShowInformationSign:
				::SetWindowText( this->GetItem( IDC_PROG_STATIC_INFO2 ), reinterpret_cast<wchar_t*>( lp ) );
				return true;
			default:
				return ModalDialog::DialogProc( msg, wp, lp );
		}
	}

	void ConvertingDialog::OnInitDialog() {
		HWND hwnd;

		m_thread = NULL;
		m_isCancelConvert = false;

		hwnd = this->GetItem( IDC_PROG_STATIC_INFO );
		::SetWindowText( hwnd, L"" );

		hwnd = this->GetItem( IDC_PROG_NOW );
		::SendMessage( hwnd, PBM_SETRANGE32, 0, static_cast<LPARAM>( m_targets.size() ) );
		::SendMessage( hwnd, PBM_SETPOS, 0, 0 );

		this->PostMsg( StartConvertSign, 0, 0 );
	}

	bool ConvertingDialog::OnOk() {
		return false;
	}

	bool ConvertingDialog::OnCancel() {
		if (m_isCancelConvert == false) {
			m_isCancelConvert = true;
			::SetWindowText( this->GetItem( IDC_PROG_STATIC_INFO ), Resource::GetString( IDS_CVT_CANCELED ).c_str() );
		}
		return false;
	}

	bool ConvertingDialog::OnCommand( WORD id, WORD code, HWND hwnd ) {
		return false;
	}

	void ConvertingDialog::OnStartConvert() {
#ifdef _DEBUG
		this->ConvertProc( this );
#else
		HANDLE thread = NULL;
		DWORD threadId;

		thread = ::CreateThread( NULL, 0, ConvertingDialog::ConvertProc, this, 0, &threadId );
		if (thread != NULL) {
			m_thread = thread;
		}
		else {
			::MessageBox( m_hwnd, L"Raised fatal error in CreateThread().", Resource::GetString( IDS_APPNAME ).c_str(), MB_OK | MB_ICONSTOP );
			::PostMessage( m_hwnd, EndDialogSign, 0, 0 );
		}
#endif
	}

	DWORD WINAPI ConvertingDialog::ConvertProc( LPVOID p ) {
#ifdef _DEBUG
#define SENDNOWINDEX( index ) _this->PostMsg( UpdateProgressSign, index, 0 )
#define SENDINFO( msg ) _this->PostMsg( ShowInformationSign, 0, reinterpret_cast<LPARAM>( msg ) )
#else
#define SENDNOWINDEX( index ) _this->SendMsg( UpdateProgressSign, index, 0 )
#define SENDINFO( msg ) _this->SendMsg( ShowInformationSign, 0, reinterpret_cast<LPARAM>( msg ) )
#endif

		ConvertingDialog *_this = reinterpret_cast<ConvertingDialog*>( p );

		wstring outputDir;
		O2JamMusic music;
		O2JamMusicDesc mdesc;
		BmsConverter bms;
		BmsConverter::FromO2Jam bdesc;
		ImageConverter img;
		MusicPackage mpack;
		SoundRanking srank;
		SoundConverter *psconv;
		int samples;
		int bits;
		int count;

		wstringstream ss;
		int soundConvFailed;

		wstring msgReady = Resource::GetString( IDS_PROGRESS_READY );
		wstring msgScore = Resource::GetString( IDS_PROGRESS_SCORE );
		wstring msgSound = Resource::GetString( IDS_PROGRESS_SOUND );
		wstring msgSongImg = Resource::GetString( IDS_PROGRESS_SONGIMG );

		//2007/6/10 1.04----->
//		switch (_this->m_format.Get()) {
//			case (SoundFormat::OggVorbis):
//				samples = 44100;
//				bits = 16;
//				psconv = new VorbisSoundConverter();
//				break;
//			case (SoundFormat::Wave):
//				switch (_this->m_quality) {
//					case LowWaveQuality:
//						samples = 22050;
//						bits = 8;
//						break;
//					case MiddleWaveQuality:
//						samples = 22050;
//						bits = 16;
//						break;
//					case HighWaveQuality:
//						samples = 44100;
//						bits = 16;
//						break;
//					default:
//						samples = 22050;
//						bits = 16;
//						break;
//				}
//				psconv = new WaveSoundConverter( samples, bits );
//				break;
//			default:
//				psconv = NULL;
//				break;
//		}
		switch (_this->m_quality) {
			case LowWaveQuality:
				samples = 22050;
				bits = 8;
				break;
			case MiddleWaveQuality:
				samples = 22050;
				bits = 16;
				break;
			case HighWaveQuality:
				samples = 44100;
				bits = 16;
				break;
			case OriginalWaveQuality:
				samples = -1;
				bits = -1;
				break;
			default:
				samples = 22050;
				bits = 16;
				break;
		}
		switch (_this->m_format.Get()) {
			case (SoundFormat::Wave):
				psconv = new WaveSoundConverter( samples, bits );
				break;
			case (SoundFormat::OggVorbis):
				psconv = new VorbisSoundConverter( samples, bits );
				break;
			default:
				psconv = NULL;
				break;
		}
		if (_this->m_quality == OriginalWaveQuality) {
			samples = 44100;
			bits = 16;
		}
		//<-------------------

		for (int i = 0; (i < static_cast<int>( _this->m_targets.size() )) && (_this->m_isCancelConvert == false); i++) {
			SENDNOWINDEX( i );
			SENDINFO( msgReady.c_str() );

			//2007/5/23 1.03----->
//			if (music.Open( _this->m_targets[i] ) == false) {
			if (music.Open( _this->m_targets[i].GetPath() ) == false) {
			//<-------------------
				_this->m_results[i] = ConvertResult::FailedToOpenSource;
				goto NextTarget;
			}

			mdesc = music.GetDescription();

			ss.str( L"" );
			ss << dec;
			ss.width( 4 );
			ss.fill( L'0' );
			ss << mdesc.GetIdentifier();
			ss << dec;
			ss << L"-" << mdesc.GetTitle();
			outputDir = Utility::CombinePath( _this->m_outputDir, ss.str() );

			//2007/5/24 1.03----->
			if (_this->m_dirExistsAbort == true && Utility::IsDirectoryExists( outputDir ) == true) {
				_this->m_results[i] = ConvertResult::DirectoryAlreadyExists;
				goto NextTarget;
			}
			//<-------------------

			if (Utility::CreateDirectory( outputDir ) == false) {
				_this->m_results[i] = ConvertResult::FailedToCreateOutDir;
				goto NextTarget;
			}

			//2007/5/23 1.03----->
//			if (mpack.Open( Utility::CombinePath( Utility::GetDirectoryFromPath( _this->m_targets[i] ), music.GetMusicFilePath() ) ) == false) {
			if (mpack.Open( Utility::CombinePath( Utility::GetDirectoryFromPath( _this->m_targets[i].GetPath() ), music.GetMusicFilePath() ) ) == false) {
			//<-------------------
				_this->m_results[i] = ConvertResult::FailedToOpenSoundFile;
				goto NextTarget;
			}

			srank.Build( mpack );

			bdesc.inputEasyNote = _this->m_outputEasy == true ? music.GetEasyScorePath() : L"";
			bdesc.inputNormalNote = _this->m_outputNormal == true ? music.GetNormalScorePath() : L"";
			bdesc.inputHardNote = _this->m_outputHard == true ? music.GetHardScorePath() : L"";
			bdesc.outputDir = outputDir;
			//08/4/20 1.05----->
			bdesc.diffType = _this->m_diffType;
			//<-----------------
			bdesc.maxLevel = _this->m_maxLevel;
			//2007/5/20 1.02----->
			bdesc.maxLevelO2 = _this->m_maxLevelO2;
			//<-------------------
			bdesc.format = _this->m_format.Get();
			bdesc.judgeRank = _this->m_judgeRank;
			bdesc.bgmCount = mpack.GetBgmCount();
			bdesc.psoundRanking = &srank;
			//2007/2/24 1.01----->
			bdesc.nothingLongNote = _this->m_nothingLongNote;
			//<-------------------
			if (_this->m_limitedSoundCount == true) {
				bdesc.dummySoundIndex = Application::DummySoundIndexLimited;
				bdesc.maxSoundCount = Application::LimitedMaxSoundCount;
			}
			else {
				bdesc.dummySoundIndex = Application::DummySoundIndexUnlimited;
				bdesc.maxSoundCount = Application::UnlimitedMaxSoundCount;
			}

			SENDINFO( msgScore.c_str() );
			if (bms.Convert( bdesc, music ) == false) {
				_this->m_results[i] = ConvertResult::FailedToConvertBms;
				goto NextTarget;
			}

			soundConvFailed = 0;
			count = min( mpack.GetItemCount(), bdesc.maxSoundCount );

			for (int j = 0; j < count; j++) {
				ss.str( L"" );
				ss << msgSound;
				ss << L" ";
				ss << j + 1 << L"/" << count;
				SENDINFO( ss.str().c_str() );

				ss.str( L"" );
				ss << AlphaDecimal( j + 1 ).ToString();
				ss << SoundFormat( _this->m_format ).GetExtension();

				if (psconv->Convert( mpack.GetItemStream( j ), Utility::CombinePath( outputDir, ss.str() ) ) == false) {
					soundConvFailed++;
				}
			}

			ss.str( L"" );
			ss << AlphaDecimal( bdesc.dummySoundIndex ).ToString();
			ss << SoundFormat( SoundFormat::Wave ).GetExtension();
			if (WaveSoundConverter::CreateMuteSound( Utility::CombinePath( outputDir, ss.str() ), samples, bits, samples / 20 ) == false) {
				soundConvFailed++;
			}

			SENDINFO( msgSongImg.c_str() );
			if (img.Convert( music.GetSongImagePath(), Utility::CombinePath( outputDir, L"songimg.bmp" ) ) == false) {
				_this->m_results[i] = ConvertResult::FailedToConvertSongImage;
				//goto NextTarget;  //̏ŌȂ̂ŁAƂ̓TEhׂĕϊĂ邩ǂɔC
			}

			if (soundConvFailed > 0) {
				_this->m_results[i] = ConvertResult::HaveAnySoundConvertError;
			}
			else if (_this->m_results[i] != ConvertResult::FailedToConvertSongImage) {
				if (bdesc.outOfRankSoundCount == 0) {
					_this->m_results[i] = ConvertResult::Okay;
				}
				else {
					_this->m_results[i] = ConvertResult::AnySoundsReplacedDummy;
				}
			}

		NextTarget:
			srank.Clear();
			mpack.Close();
			music.Close();
		}

		_this->PostMsg( EndDialogSign, 0, 0 );
		return 0;
	}
}
}