#include "stdafx.h"
#include "FileDialog.hpp"
#include "Utility.hpp"

namespace Lmt {
namespace O2Bms {
	FileDialog::FileDialog() {
		m_parent = NULL;
		m_title = L"";
		m_initialDir = L".";
		m_initialFile = L"*";
		m_filter = L"All Files(*)|*";
		m_defExt = L"";
		m_multiSelect = false;
		this->BuildFilterNative();
	}

	int FileDialog::GetCount() const {
		return static_cast<int>( m_files.size() );
	}

	wstring FileDialog::GetFileName( int index ) const {
		return m_files[index];
	}

	void FileDialog::SetParent( HWND parent ) {
		m_parent = parent;
	}

	void FileDialog::SetTitle( const wstring &title ) {
		m_title = title;
	}

	void FileDialog::SetInitialDir( const wstring &dir ) {
		m_initialDir = dir;
	}

	void FileDialog::SetInitialFile( const wstring &file ) {
		m_initialFile = file;
	}

	void FileDialog::SetFilter( const wstring &filter ) {
		m_filter = filter;
		this->BuildFilterNative();
	}

	void FileDialog::SetDefaultExtension( const wstring &defExt ) {
		m_defExt = defExt;
	}

	void FileDialog::SetMultiSelect( bool multiSelect ) {
		m_multiSelect = multiSelect;
	}


	bool FileDialog::SelectOpen() {
		OPENFILENAME ofn;
		const int bufFileLength = 256 * 1024 + 65535;
		const int bufFileTitleLength = MAX_PATH + 10;
		vector<wchar_t> bufferFile( bufFileLength );
		vector<wchar_t> bufferFileTitle( bufFileTitleLength );

		memset( &ofn, 0, sizeof(ofn) );
		wcscpy( &(bufferFile.at( 0 )), m_initialFile.c_str() );
		wcscpy( &(bufferFileTitle.at( 0 )), L"" );

		ofn.lStructSize = sizeof(ofn);
		ofn.hwndOwner = m_parent;
		ofn.lpstrFilter = m_filterNative.c_str();
		ofn.nFilterIndex = 0;
		ofn.lpstrFile = &(bufferFile.at( 0 ));
		ofn.nMaxFile = bufFileLength;
		ofn.lpstrFileTitle = &(bufferFileTitle.at( 0 ));
		ofn.nMaxFileTitle = bufFileTitleLength;
		ofn.lpstrInitialDir = m_initialDir.c_str();
		ofn.lpstrTitle = m_title.c_str();
		ofn.lpstrDefExt = m_defExt.c_str();
		ofn.Flags = OFN_HIDEREADONLY | OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_EXPLORER | OFN_LONGNAMES | OFN_ENABLESIZING;
		ofn.Flags |= (m_multiSelect == true) ? OFN_ALLOWMULTISELECT : 0;
		if (!::GetOpenFileName( &ofn )) {
			return false;
		}

		wstring path = &(bufferFile.at( 0 ));
		wchar_t *pfile = (&(bufferFile.at( 0 ))) + path.length() + 1;

		m_files.clear();
		if (!(*pfile && m_multiSelect == true)) {
			m_files.push_back( path );
		}
		else {
			size_t fileLength;
			while (*pfile) {
				fileLength = wcslen( pfile );
				m_files.push_back( Utility::CombinePath( path, wstring( pfile, fileLength ) ) );
				pfile += (fileLength + 1);
			}
		}

		return true;
	}

	bool FileDialog::SelectSave() {
		OPENFILENAME ofn;
		const int bufFileLength = MAX_PATH + 10;
		const int bufFileTitleLength = MAX_PATH + 10;
		vector<wchar_t> bufferFile( bufFileLength );
		vector<wchar_t> bufferFileTitle( bufFileTitleLength );

		memset( &ofn, 0, sizeof(ofn) );
		wcscpy( &(bufferFile.at( 0 )), m_initialFile.c_str() );
		wcscpy( &(bufferFileTitle.at( 0 )), L"" );

		ofn.lStructSize = sizeof(ofn);
		ofn.hwndOwner = m_parent;
		ofn.lpstrFilter = m_filterNative.c_str();
		ofn.nFilterIndex = 0;
		ofn.lpstrFile = &(bufferFile.at( 0 ));
		ofn.nMaxFile = bufFileLength;
		ofn.lpstrFileTitle = &(bufferFileTitle.at( 0 ));
		ofn.nMaxFileTitle = bufFileTitleLength;
		ofn.lpstrInitialDir = m_initialDir.c_str();
		ofn.lpstrTitle = m_title.c_str();
		ofn.lpstrDefExt = m_defExt.c_str();
		ofn.Flags = OFN_HIDEREADONLY | OFN_PATHMUSTEXIST | OFN_EXPLORER | OFN_LONGNAMES | OFN_ENABLESIZING;
		if (!::GetSaveFileName( &ofn )) {
			return false;
		}

		m_files.clear();
		m_files.push_back( &(bufferFile.at( 0 )) );
		return true;
	}

	void FileDialog::ClearFileBuffer() {
		m_files.clear();
	}

	void FileDialog::BuildFilterNative() {
		m_filterNative = m_filter;
		m_filterNative += L"||";
		for (wstring::iterator it = m_filterNative.begin(); it != m_filterNative.end(); it++) {
			if (*it == L'|') {
				*it = L'\0';
			}
		}
	}
}
}
