#include "stdafx.h"
#include "ModalDialog.hpp"

namespace Lmt {
namespace O2Bms {
	ModalDialog::ModalDialog( int id ) {
		m_hwnd = NULL;
		m_id = id;
	}

	HWND ModalDialog::GetHandle() const {
		return m_hwnd;
	}
	
	HWND ModalDialog::GetItem( int id ) const {
		return ::GetDlgItem( m_hwnd, id );
	}

	int ModalDialog::Run( HWND parent ) {
		return static_cast<int>(
			::DialogBoxParam(
				reinterpret_cast<HINSTANCE>( ::GetModuleHandle( NULL ) ),
				MAKEINTRESOURCE( m_id ),
				parent,
				reinterpret_cast<DLGPROC>( ModalDialog::DialogProcBase ),
				reinterpret_cast<LPARAM>( this )
			)
		);
	}

	LRESULT ModalDialog::SendMsg( UINT msg, WPARAM wp, LPARAM lp ) const {
		return ::SendMessage( m_hwnd, msg, wp, lp );
	}

	LRESULT ModalDialog::PostMsg( UINT msg, WPARAM wp, LPARAM lp ) const {
		return ::PostMessage( m_hwnd, msg, wp, lp );
	}

	LRESULT CALLBACK ModalDialog::DialogProcBase( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp ) {
		ModalDialog  *pdialog = reinterpret_cast<ModalDialog*>( ::GetWindowLong( hwnd, GWL_USERDATA ) );

		if (pdialog != NULL) {
			switch (msg) {
				case WM_COMMAND:
					switch ( LOWORD( wp ) ) {
						case IDOK:
							if (pdialog->OnOk() == true) {
								::EndDialog( hwnd, IDOK );
								return true;
							}
							return false;
						case IDCANCEL:
							if (pdialog->OnCancel() == true) {
								::EndDialog( hwnd, IDCANCEL );
								return true;
							}
							return false;
						default:
							return pdialog->OnCommand( LOWORD( wp ), HIWORD( wp ), reinterpret_cast<HWND>( lp ) );
					}
				default:
					return pdialog->DialogProc( msg, wp, lp );
			}
		}

		if (msg == WM_INITDIALOG) {
			::SetWindowLong( hwnd, GWL_USERDATA, static_cast<LONG>( lp ) );
			pdialog = reinterpret_cast<ModalDialog*>( lp );
			pdialog->m_hwnd = hwnd;
			pdialog->OnInitDialog();
			return true;
		}

		return 0;
	}

	bool ModalDialog::DialogProc( UINT msg, WPARAM wp, LPARAM lp ) {
		return false;
	}

	void ModalDialog::OnInitDialog() {
	}

	bool ModalDialog::OnOk() {
		return true;
	}

	bool ModalDialog::OnCancel() {
		return true;
	}

	bool ModalDialog::OnCommand( WORD id, WORD code, HWND hwnd ) {
		return false;
	}

}
}
