#include "stdafx.h"
#include "MusicPackage.hpp"
#include "Utility.hpp"
//08/4/20 1.05----->
#include "OjmPackageFlags.hpp"
//<-----------------

namespace Lmt {
namespace O2Bms {
	MusicPackage::MusicPackage() {
		m_pfs = NULL;
	}

	MusicPackage::~MusicPackage() {
		this->Close();
	}

	bool MusicPackage::Open( const wstring &path ) {
		if (m_pfs != NULL) {
			return false;
		}

		FileHeader header;
		StreamArray streams;
		StreamArray::iterator itStream;
		fstream *pfs = new fstream( Utility::WStringToString( path ).c_str(), ios::in | ios::binary );
		int bgmCount = 0;
		int fileLength;
		int rank;

		if (pfs->is_open() == false) {
			goto ErrorFunc;
		}

		pfs->seekg( 0, ios::end );
		fileLength = pfs->tellg();
		pfs->seekg( 0, ios::beg );

		pfs->read( (char*)&header, sizeof(header) );
		if (pfs->gcount() != sizeof(header)) {
			goto ErrorFunc;
		}

		if (memcmp( header.Signature, "M30\0", 4 ) != 0) {
			goto ErrorFunc;
		}
		if (header.Length != (fileLength - sizeof(header))) {
			goto ErrorFunc;
		}

		while (pfs->tellg() < fileLength) {
			//08/4/20 1.05----->
//			itStream = streams.insert( streams.end(), MusicPackageStream( pfs ) );
			itStream = streams.insert( streams.end(), MusicPackageStream( pfs, ((header.Flags & OjmEncrypted) != 0) ? true : false ) );
			//<-----------------
			if (itStream->Available() == false) {
				goto ErrorFunc;
			}
			if (itStream->IsBackgroundMusic() == true) {
				bgmCount++;
			}
		}

		std::sort( streams.begin(), streams.end(), greater<MusicPackageStream>() );
		for (itStream = streams.begin(), rank = 1; itStream != streams.end(); itStream++, rank++) {
			itStream->m_rank = rank;
		}

		m_header = header;
		m_streams.swap( streams );
		m_pfs = pfs;
		m_bgmCount = bgmCount;
		return true;

	ErrorFunc:
		if (pfs != NULL) {
			pfs->close();
			delete pfs;
		}
		return false;
	}

	void MusicPackage::Close() {
		if (m_pfs != NULL) {
			m_pfs->close();
			m_streams.clear();
			memset( &m_header, 0, sizeof(m_header) );
			m_pfs = NULL;
		}
	}

	int MusicPackage::GetSamplesPerSecBps() const {
		return m_header.SamplesPerSec;
	}

	//08/04/20 1.05----->
//	int MusicPackage::GetBitsPerSample() const {
//		return m_header.BitsPerSample;
//	}
	int MusicPackage::GetFlags() const {
		return m_header.Flags;
	}
	//<------------------

	int MusicPackage::GetBgmCount() const {
		return m_bgmCount;
	}

	int MusicPackage::GetItemCount() const {
		return static_cast<int>( m_streams.size() );
	}

	MusicPackageStream * MusicPackage::GetItemStream( int index ) {
		return &(m_streams.at( index ));
	}

	const MusicPackageStream * MusicPackage::GetItemStream( int index ) const {
		return &(m_streams.at( index ));
	}
}
}
