#include "stdafx.h"
#include "SoundRanking.hpp"

namespace Lmt {
namespace O2Bms {
	int SoundRanking::GetCount() const {
		return m_ranking.size();
	}

	int SoundRanking::GetRank( int identifier ) const {
		RankingMap::const_iterator it = m_ranking.find( identifier );
		return it == m_ranking.end() ? NotFound : it->second;
	}

	void SoundRanking::Build( const MusicPackage &mpack ) {
		RankingMap ranking;
		const MusicPackageStream *pmstream;

		for (int i = 0; i < mpack.GetItemCount(); i++) {
			pmstream = mpack.GetItemStream( i );
			if (pmstream->IsBackgroundMusic() == true) {
				ranking.insert( make_pair( pmstream->GetIdentifier(), pmstream->GetRank() ) );
			}
			else {
				ranking.insert( make_pair( pmstream->GetIdentifier() + mpack.GetBgmCount(), pmstream->GetRank() ) );
			}
		}

		m_ranking.swap( ranking );
	}

	void SoundRanking::Clear() {
		m_ranking.clear();
	}

#ifdef _DEBUG
	struct mapdata {
		mapdata( int _id, int _rank ) : id( _id ), rank( _rank ){}
		int id, rank;
	};
	void SoundRanking::MakeArray() const {
		vector<mapdata> ar;
		for (RankingMap::const_iterator it = m_ranking.begin(); it != m_ranking.end(); it++) {
			ar.push_back( mapdata( it->second, it->first ) );
		}
		return;
	}
#endif
}
}
